﻿using DxLibDLL;
using System;

namespace Tetris
{
    /// <summary>
    /// ブロック描画クラス
    /// プレイヤーが操作するピースとフィールドのブロックの描画処理を
    /// 共通化する目的で使用する。
    /// </summary>
    class BlockDrawer
    {
        // 描画コントローラなのでSingletonで実装する
        // （実際のところ、Singletonで実装する必要性はないと思う）
        private static BlockDrawer instance = new BlockDrawer();
        
        private BlockDrawer() { }

        public static BlockDrawer Instance
        {
            get { return instance; }
        }

        /// <summary>
        /// 描画処理。指定した座標に指定した種類のブロックを描画する。
        /// </summary>
        /// <param name="x">描画する座標 X （フィールドの座標）</param>
        /// <param name="y">描画する座標 Y （フィールドの座標）</param>
        /// <param name="kind">描画するブロックの種類</param>
        public void Draw(int x, int y, int kind)
        {
            // ブロックの色
            int colorBase = 0;
            int colorBright = 0;
            int colorShade = 0;

            // 種別により色をセット
            switch (kind)
            {
                case 0:
                    break;
                case 1: // I型
                    colorBright = 0x66ffff;
                    colorBase = 0x33ccff;
                    colorShade = 0x0099cc;
                    break;
                case 2: // O型
                    colorBright = 0xffff99;
                    colorBase = 0xffcc33;
                    colorShade = 0xcc9933;
                    break;
                case 3: // J型
                    colorBright = 0x99ccff;
                    colorBase = 0x3366cc;
                    colorShade = 0x000099;
                    break;
                case 4: // L型
                    colorBright = 0xffcc66;
                    colorBase = 0xff9900;
                    colorShade = 0xcc6600;
                    break;
                case 5: // S型
                    colorBright = 0xccff66;
                    colorBase = 0x99ff33;
                    colorShade = 0x336633;
                    break;
                case 6: // Z型
                    colorBright = 0xffcc99;
                    colorBase = 0xff6633;
                    colorShade = 0x990000;
                    break;
                case 7: // T型
                    colorBright = 0xff99ff;
                    colorBase = 0xff33ff;
                    colorShade = 0x660066;
                    break;
                case 8:
                    colorBright = 0xffffff;
                    colorBase = 0xcccccc;
                    colorShade = 0x666666;
                    break;
            }

            // 描画の基準になる座標を計算する
            int lx = (GL.ScrX - (GL.FldX * GL.BlkSiz)) / 2;
            int ly = (GL.ScrY - (GL.FldY * GL.BlkSiz)) / 2 + GL.BlkSiz;

            // ブロックを描画する座標を計算する
            int dx = lx + (x * GL.BlkSiz);
            int dy = ly + (y * GL.BlkSiz);

            // 描画する
            DX.DrawBox(dx + 1 + 0, dy + 1 + 0, dx + (GL.BlkSiz - 1) - 0, dy + (GL.BlkSiz - 1) - 0, colorBright, DX.TRUE);
            DX.DrawBox(dx + 1 + 2, dy + 1 + 2, dx + (GL.BlkSiz - 1) - 0, dy + (GL.BlkSiz - 1) - 0, colorShade, DX.TRUE);
            DX.DrawBox(dx + 1 + 2, dy + 1 + 2, dx + (GL.BlkSiz - 1) - 2, dy + (GL.BlkSiz - 1) - 2, colorBase, DX.TRUE);
        }
    }
}
