﻿using DxLibDLL;
using System;

namespace Tetris
{
    /// <summary>
    /// フィールドデータを管理するクラス
    /// </summary>
    class Field
    {
        // フィールドデータ
        // ０：何もない　１～７：ブロック　－１：壁
        private int[,] fieldData = new int[GL.FldX, GL.FldY];

        /// <summary>
        /// 指定の座標に値を設定する
        /// </summary>
        /// <param name="x">X軸座標（０～　９）</param>
        /// <param name="y">Y軸座標（０～１９）</param>
        /// <param name="data">設定するデータ</param>
        public void Set(int x, int y, int data)
        {
            // 範囲外にデータを設定しようとした場合は
            // 設定しないで戻る
            if (x < 0) return;
            if (x > GL.FldX - 1) return;
            if (y < 0) return;
            if (y > GL.FldY - 1) return;

            // 値をセット
            fieldData[x, y] = data;
        }

        /// <summary>
        /// 指定の座標の値を取得する
        /// </summary>
        /// <param name="x">X軸座標（０～　９）</param>
        /// <param name="y">Y軸座標（０～１９）</param>
        /// <returns>データ</returns>
        public int Get(int x, int y)
        {
            // 範囲外のデータを取得しようとした場合は
            // 移動不可を表す値を返す
            if (x < 0) return -1;
            if (x > GL.FldX - 1) return -1;
            if (y > GL.FldY - 1) return -1;
            // 中空は何もない領域と見なす
            if (y < 0) return 0;

            // 値を返す
            return fieldData[x, y];
        }

        /// <summary>
        /// 削除処理
        /// </summary>
        public void Shave()
        {
            // 一行ずつチェックする
            for (int y = 0; y < GL.FldY; y++)
            {
                // 隙間を検索する
                bool f = true;
                for (int x = 0; x < GL.FldX; x++)
                {
                    if (fieldData[x, y] == 0)
                    {
                        // 隙間があった場合はフラグを落とす
                        f = false;
                    }
                }
                // 隙間が無かった場合
                if (f)
                {
                    // そのラインを削除する
                    for (int x = 0; x < GL.FldX; x++)
                    {
                        // 削除
                        fieldData[x, y] = 0;

                        // そのラインより上のラインを落下（一段下にずらす）させる
                        for (int y2 = y - 1; y2 > 0; y2--)
                        {
                            fieldData[x, y2 + 1] = fieldData[x, y2];
                            fieldData[x, y2] = 0;
                        }
                    }
                }
            }

            // 今の状態では削除して即落下になってしまう
            // 削除している間のアニメーション処理などを追加すると
            // それっぽくなる
        }

        /// <summary>
        /// 描画処理
        /// </summary>
        public void Draw()
        {
            // フィールドの値を描画する
            for (int x = 0; x < GL.FldX; x++)
            {
                for (int y = 0; y < GL.FldY; y++)
                {
                    BlockDrawer block = BlockDrawer.Instance;
                    block.Draw(x, y, fieldData[x, y]);
                }
            }
        }
    }
}
