﻿using System;

namespace Tetris
{
    /// <summary>
    /// Nextピース（次に出る「テトリミノ」）を管理するクラス
    /// Nextピースの偏りを無くすための処理を行う。
    /// 本当はARIKAのTGMのようにしたいが、複雑になるので簡略化、
    /// いわゆる「ワールドルール」「ガイドライン対応版」などと呼ばれるテトリスで
    /// 行っている程度の処理にする。
    /// </summary>
    class NextPiece
    {
        // 乱数生成用
        Random rand = new Random();

        // Nextピースの保存用
        int[] nextes = new int[14];
        int cursor = 0;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public NextPiece()
        {
            // ちょっと強引だが、これで１４個分生成する
            Reset();
            Reset();
        }

        /// <summary>
        /// テーブルのリセット
        /// 後半７個分を、前半７個に移す。後半７個分をランダム生成。
        /// </summary>
        private void Reset()
        {
            // 生成用のテーブル
            int[] tbl = new int[7];
            // テーブルに１～７をセット
            for (int i = 0; i < 7; i++)
            {
                tbl[i] = i + 1;
            }
            // テーブルをシャッフル
            for (int i = 0; i < 7; i++)
            {
                int r = rand.Next(7);
                int l = rand.Next(7);

                int w = tbl[r];
                tbl[r] = tbl[l];
                tbl[l] = w;
            }
            // Nextピースの後半を前半に複製
            for (int i = 0; i < 7; i++)
            {
                nextes[i] = nextes[i + 7];
            }
            // Nextピースの後半に生成したテーブルをセット
            for (int i = 0; i < 7; i++)
            {
                nextes[7 + i] = tbl[i];
            }
            // カーソル位置をリセット
            cursor = 0;
        }

        /// <summary>
        /// 次の１ピースを取得する
        /// </summary>
        /// <returns>１ピース</returns>
        public int Next()
        {
            // カーソル位置が後半に行ったときはリセット
            if (cursor >= 7)
            {
                Reset();
            }

            // 値を返す
            return nextes[cursor++];
        }
    }
}
